package Servlet;

import com.alibaba.fastjson.JSON;
import lianxipojo.User;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 用户管理servlet
 */
@WebServlet("/user.do")
public class UserServlet extends HttpServlet {

    //获取页面初始化数据
    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        this.doPost(req, resp);
    }
    //doPost中进行判断前端传回来的值
    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String flag = req.getParameter("flag");
        if (flag.equals("getData")){
            getData(req,resp);
        }else if (flag.equals("addUser")){
            addUser(req,resp);
        }else if (flag.equals("updateUser")){
            updateUser(req,resp);
        }
        super.doPost(req, resp);
    }



    //初始化用户数据
    @Override
    public void init() throws ServletException {
        ArrayList<User> list = new ArrayList<>();
        User user1 = new User("1","hxz","123456","2001-01-01",30000.00);
        User user2 = new User("2","lzq","12","2001-01-01",30000.00);
        User user3 = new User("3","mby","123","2001-01-01",3000.00);
        User user4 = new User("4","zsx","1234","2001-01-01",30000.00);
        User user5 = new User("5","lhh","12345","2001-01-01",30000.00);
        User user6 = new User("6","wzh","6","2001-01-01",3000.00);
        list.add(user1);
        list.add(user2);
        list.add(user3);
        list.add(user4);
        list.add(user5);
        list.add(user6);
        //利用ServletContext对象来获取上下文
        ServletContext servletContext = this.getServletContext();
        servletContext.setAttribute("list",list);//设置属性
    }
    /**
     * 获取用户数据
     * @param req
     * @param resp
     */
    private void getData(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //从getServletContext（）方法中把List属性取出来，取出来后根据所需类型强转
        List<User> list = (List<User>) this.getServletContext().getAttribute("list");
        String s = JSON.toJSONString(list);
        resp.setContentType("application/json");
        PrintWriter pw = resp.getWriter();
        pw.print(s);
        pw.flush();
        pw.close();

    }
    /**
     * 添加用户
     * @param req
     * @param resp
     */
    private void addUser(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //获取User对象
        User user = this.createUser(req);
        ServletContext servletContext = this.getServletContext();
        //将之前已有的list拿出来
        List<User> list = (List<User>) servletContext.getAttribute("list");
        list.add(user);
        resp.setContentType("text/html;charset=UTF-8");
        PrintWriter printWriter = resp.getWriter();
        printWriter.println("添加成功！");
        printWriter.flush();
        printWriter.close();
    }

    /**
     * 创建用户
     * @param request
     * @return
     */
    private User createUser(HttpServletRequest request){
        String userid = request.getParameter("userid");
        String username = request.getParameter("userName");
        String userpassword = request.getParameter("userPassword");
        String salary = request.getParameter("salary");
        String birthday = request.getParameter("birthday");


        //将值设置进去
        User user = new User();
        user.setId(userid);
        user.setUserName(username);
        user.setUserPassword(userpassword);
        user.setSalary(Double.parseDouble(salary));
        user.setBirthday(birthday);

        return user;
    }

    /**
     * 更新用户的数据，先删后插
     * @param req
     * @param resp
     */
    private void updateUser(HttpServletRequest req, HttpServletResponse resp) throws IOException {
        User user = this.createUser(req);
        ServletContext servletContext = getServletContext();
        List<User> list = (List<User>) servletContext.getAttribute("list");
        //可遍历list找到所需对象，也可将list转换成map，从而根据key值找到
        //用list.stream().collect(Collectors.toMap(User::getId, Function.identity()))可将list转换成map
        Map<String,User> map = list.stream().collect(Collectors.toMap(User::getId, Function.identity()));
        //根据id查找相对应的对象
        User user1 = map.get(user.getId());
        list.remove(user1);
        //
        list.add(user);
        list.sort(new Comparator<User>() {
            @Override
            public int compare(User o1, User o2) {
                return Integer.parseInt(o1.getId())-Integer.parseInt(o2.getId());
            }
        });
        //输出页面
        resp.setContentType("text/html;charset=UTF-8");
        PrintWriter printWriter = resp.getWriter();
        printWriter.println("更新成功！");
        printWriter.flush();
        printWriter.close();
    }



}
